/*
 Standalone Current Logger for ATtiny84
 Version: 0.3
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 25/04/2012
 Last Modified: 29/05/2012
 
 This is a test to use an ATtiny84 with an OPA333 op-amp to log the current passing through a load.
 Logs 150 readings in under 25ms and transfers it to your PC. Needs to be powered by USB.
 Press the button to begin.
 
 Remember to change your fuse bits so the ATtiny84 uses 16MHz crystal and divide clock by 8
 avrdude -p ATtiny84 -c usbtiny -U lfuse:w:0x7f:m -U hfuse:w:0xdf:m -U efuse:w:0xff:m
 
 */

// ATtiny24/44/84 Pin map
//
//               +-\/-+
//         VCC  1|o   |14  GND
//         PB0  2|    |13  PA0 (ADC0)
//         PB1  3|    |12  PA1 (ADC1)
//   Reset PB3  4|    |11  PA2 (ADC2)
//         PB2  5|    |10  PA3 (ADC3)
//  (ADC7) PA7  6|    |9   PA4 (ADC4)
//  (ADC6) PA6  7|    |8   PA5 (ADC5)
//               +----+
//

#define adcPin 3 // Analog input 3
#define ledPin PA5 // Output
#define buttonPin PA7 // Input
#define usbminusPin PA1 // USB D-

#define F_CPU 16000000 // If this will be changed, modify it in usbconfig.h too

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <math.h>
#include <util/delay.h>
#include "usbdrv/usbdrv.h"
#include "usbdrv/oddebug.h"
#include "setup.c"

int adcMemory[150];

// Convert a number to HID values and add it to the nextDigit variable
void numberTohid(int number) {
	uchar digit;
	do {
		digit = number % 10;
		number /= 10;
		*--nextDigit = 0;
		if (digit == 0) {
			*--nextDigit = KEY_0;
		}
		else {
			*--nextDigit = KEY_1 - 1 + digit;
		}
	} while (number != 0);
}

// Send the next digit from the address
int sendnextDigit(uint16_t eepromAddress) {
	int value = adcMemory[eepromAddress];
	
	int baseline = 13; // 13 * 0.001074 = ~14mV baseline
	double calculateDouble = ((double) (value - baseline) * 0.01074);

	// Split the value into 2 parts
	int beforeDecimalpoint = (int) calculateDouble; // Before the decimal point
	int afterDecimalpoint = (int) ((calculateDouble - beforeDecimalpoint) * 1000); // After the decimal point
	
	// Fixes incorrect rounding after decimal point
	bool insertNumberzero = false;
	if (afterDecimalpoint < 100) {
		insertNumberzero = true;
	}
	
	nextDigit = &valueBuffer[sizeof(valueBuffer)];
	*--nextDigit = 0xff;
	*--nextDigit = 0;
	*--nextDigit = KEY_RETURN;
	
	if (afterDecimalpoint != 0) {
		numberTohid(afterDecimalpoint);
		
		if (insertNumberzero == true) {
			*--nextDigit = 0;
			*--nextDigit = KEY_0;
		}
		
		// Dot
		*--nextDigit = 0;
		*--nextDigit = KEY_DOT;
	}
	
	numberTohid(beforeDecimalpoint);
	
	/* DEBUGGING
	nextDigit = &valueBuffer[sizeof(valueBuffer)];
	*--nextDigit = 0xff;
	*--nextDigit = 0;
	*--nextDigit = KEY_RETURN;
	numberTohid(value);*/
	
	return 0;
}

// Transfers the data to USB
void transferData (void) {
	boolean inusbTransfer = true;
	uint16_t readCount = 0;
	uint16_t endCount = 150;
	
	while (inusbTransfer == true) {
		usbPoll();
		if(usbInterruptIsReady() && nextDigit != NULL){ // We can send another digit
			buildReport();
			usbSetInterrupt(reportBuffer, sizeof(reportBuffer));
			if(*++nextDigit == 0xff) { // This was terminator character
				nextDigit = NULL;
			}
		}
		if (nextDigit == NULL) {
			if (readCount < endCount) {
				int checkforZero = sendnextDigit(readCount);
				if (checkforZero == true) { // End if 0
					inusbTransfer = false;
				}
				readCount++;
			}
			else {
				inusbTransfer = false;
			}
		}
	}
}

// SPI Transfer
uint8_t spi_transfer(uint8_t data) {
	USIDR = data;
	USISR = _BV(USIOIF); // clear flag

	while ( (USISR & _BV(USIOIF)) == 0 ) {
		USICR = (1<<USIWM0)|(1<<USICS1)|(1<<USICLK)|(1<<USITC);
	}
	return USIDR;
}

int main (void) {
	setup();
	
	//DDRA |= (1<<ledPin); // Set Outputs
	DDRA &= ~(1<<usbminusPin); // Set Inputs
	//DDRA &= ~(1<<buttonPin);
	
	// Pull ups to save power
	PORTA |= ((1<<PA2) | (1<<PA4) | (1<<PA6)); 
	PORTA |= (1<<buttonPin);
	PORTA |= (1<<ledPin);
	
	// Pin interrupt setup
	sbi(GIMSK,PCIE0); // Enable pin change interrupt
	sbi(PCMSK0,PCINT7); // Apply interrupt to button pin
	
	while(1) {	
		
		int adcResult = 0;
		int adcCount = 0;
		uint8_t adcLowbyte, adcHighbyte;
		
		// Turn on LED and ADC
		PORTA |= (1<<ledPin);
		sbi(ADCSRA, ADEN);
		sbi(ADCSRA, ADIE);
		
		// Start the ADC conversion and wait to be woken up by the ADC interrupt
		set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
		sleep_mode(); // System sleeps here
		
		// Log ADC 150 times
		while (adcCount < 150) {
			// Read ADC bytes
			adcLowbyte = ADCL;
			adcHighbyte = ADCH;
			
			// Combine the bytes
			adcResult = (adcHighbyte << 8) | adcLowbyte;
			
			//Start the ADC conversion and write the previous ADC result to memory
			adcMemory[adcCount] = adcResult;
			adcCount++;
			
			// Wait to be woken up by the ADC interrupt
			set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
			sleep_mode(); // System sleeps here
		}
		
		// Turn off LED and ADC
		PORTA &= ~(1<<ledPin);
		cbi(ADCSRA, ADSC);
		cbi(ADCSRA, ADEN);
		cbi(ADCSRA, ADIE);
		
		// Increase clock speed to 16MHz
		CLKPR = (1<<CLKPCE); // Prescaler enable
		CLKPR = 0; // Clock division factor 1 (0000)
		
		// Transfer the data to the PC
		startUSB();
		transferData();
		
		// Decrease clock speed to 4MHz
		CLKPR = (1<<CLKPCE); // Prescaler enable
		CLKPR = (1<<CLKPS1); // Clock division factor 4 (0010)
	}
	
	return 0;
}

// Pin change interrupt
ISR(PCINT0_vect) { }

// ADC conversion complete interrupt
ISR(ADC_vect) { }
